
From: Dominik Brodowski <linux@dominikbrodowski.net>

Always rescan the devices upon echo'ing something to
available_resources_setup_done.  This is needed for proper "coldplug" support.

Signed-off-by: Dominik Brodowski <linux@dominikbrodowski.net>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/drivers/pcmcia/cs_internal.h  |    3 --
 25-akpm/drivers/pcmcia/ds.c           |   38 +++++++++++++++++++++++++++++++---
 25-akpm/drivers/pcmcia/socket_sysfs.c |   26 +++++++++--------------
 3 files changed, 47 insertions(+), 20 deletions(-)

diff -puN drivers/pcmcia/cs_internal.h~pcmcia-rescan-bus-always-upon-echoing-into-setup_done drivers/pcmcia/cs_internal.h
--- 25/drivers/pcmcia/cs_internal.h~pcmcia-rescan-bus-always-upon-echoing-into-setup_done	2005-03-20 16:56:47.000000000 -0800
+++ 25-akpm/drivers/pcmcia/cs_internal.h	2005-03-20 16:56:47.000000000 -0800
@@ -159,8 +159,7 @@ int pccard_access_configuration_register
 struct pcmcia_callback{
 	struct module	*owner;
 	int		(*event) (struct pcmcia_socket *s, event_t event, int priority);
-	int		(*resources_done) (struct pcmcia_socket *s);
-	void		(*replace_cis) (void);
+	void		(*requery) (struct pcmcia_socket *s);
 };
 
 int pccard_register_pcmcia(struct pcmcia_socket *s, struct pcmcia_callback *c);
diff -puN drivers/pcmcia/ds.c~pcmcia-rescan-bus-always-upon-echoing-into-setup_done drivers/pcmcia/ds.c
--- 25/drivers/pcmcia/ds.c~pcmcia-rescan-bus-always-upon-echoing-into-setup_done	2005-03-20 16:56:47.000000000 -0800
+++ 25-akpm/drivers/pcmcia/ds.c	2005-03-20 16:56:47.000000000 -0800
@@ -717,9 +717,42 @@ static inline void pcmcia_add_pseudo_dev
 	return;
 }
 
-static void pcmcia_bus_rescan(void)
+static int pcmcia_requery(struct device *dev, void * _data)
 {
+	struct pcmcia_device *p_dev = to_pcmcia_dev(dev);
+	if (!p_dev->dev.driver)
+		pcmcia_device_query(p_dev);
+
+	return 0;
+}
+
+static void pcmcia_bus_rescan(struct pcmcia_socket *skt)
+{
+	int no_devices=0;
+	unsigned long flags;
+
 	/* must be called with skt_sem held */
+	spin_lock_irqsave(&pcmcia_dev_list_lock, flags);
+	if (list_empty(&skt->pcmcia->devices_list))
+		no_devices=1;
+	spin_unlock_irqrestore(&pcmcia_dev_list_lock, flags);
+
+	/* if no devices were added for this socket yet because of
+	 * missing resource information or other trouble, we need to
+	 * do this now. */
+	if (no_devices) {
+		int ret = pcmcia_card_add(skt);
+		if (ret)
+			return;
+	}
+
+	/* some device information might have changed because of a CIS
+	 * update or because we can finally read it correctly... so
+	 * determine it again, overwriting old values if necessary. */
+	bus_for_each_dev(&pcmcia_bus_type, NULL, NULL, pcmcia_requery);
+
+	/* we re-scan all devices, not just the ones connected to this
+	 * socket. This does not matter, though. */
 	bus_rescan_devices(&pcmcia_bus_type);
 }
 
@@ -1860,8 +1893,7 @@ static int __devinit pcmcia_bus_add_sock
 	/* Set up hotline to Card Services */
 	s->callback.owner = THIS_MODULE;
 	s->callback.event = &ds_event;
-	s->callback.resources_done = &pcmcia_card_add;
-	s->callback.replace_cis = &pcmcia_bus_rescan;
+	s->callback.requery = &pcmcia_bus_rescan;
 	socket->pcmcia = s;
 
 	ret = pccard_register_pcmcia(socket, &s->callback);
diff -puN drivers/pcmcia/socket_sysfs.c~pcmcia-rescan-bus-always-upon-echoing-into-setup_done drivers/pcmcia/socket_sysfs.c
--- 25/drivers/pcmcia/socket_sysfs.c~pcmcia-rescan-bus-always-upon-echoing-into-setup_done	2005-03-20 16:56:47.000000000 -0800
+++ 25-akpm/drivers/pcmcia/socket_sysfs.c	2005-03-20 16:56:47.000000000 -0800
@@ -163,24 +163,20 @@ static ssize_t pccard_store_resource(str
 		return -EINVAL;
 
 	spin_lock_irqsave(&s->lock, flags);
-	if (!s->resource_setup_done) {
+	if (!s->resource_setup_done)
 		s->resource_setup_done = 1;
-		spin_unlock_irqrestore(&s->lock, flags);
+	spin_unlock_irqrestore(&s->lock, flags);
 
-		down(&s->skt_sem);
-		if ((s->callback) &&
-		    (s->state & SOCKET_PRESENT) &&
-		    !(s->state & SOCKET_CARDBUS)) {
-			if (try_module_get(s->callback->owner)) {
-				s->callback->resources_done(s);
-				module_put(s->callback->owner);
-			}
+	down(&s->skt_sem);
+	if ((s->callback) &&
+	    (s->state & SOCKET_PRESENT) &&
+	    !(s->state & SOCKET_CARDBUS)) {
+		if (try_module_get(s->callback->owner)) {
+			s->callback->requery(s);
+			module_put(s->callback->owner);
 		}
-		up(&s->skt_sem);
-
-		return count;
 	}
-	spin_unlock_irqrestore(&s->lock, flags);
+	up(&s->skt_sem);
 
 	return count;
 }
@@ -315,7 +311,7 @@ static ssize_t pccard_store_cis(struct k
 		if ((s->callback) && (s->state & SOCKET_PRESENT) &&
 		    !(s->state & SOCKET_CARDBUS)) {
 			if (try_module_get(s->callback->owner)) {
-				s->callback->replace_cis();
+				s->callback->requery(s);
 				module_put(s->callback->owner);
 			}
 		}
_
